USE tempdb;
GO

-- prepare test data
IF OBJECT_ID(N'dbo.Clients') IS NOT NULL
	DROP TABLE dbo.Clients;

CREATE TABLE dbo.Clients 
(
	[Id] int not null identity,
	[FirstName] nvarchar(20) not null,
	[LastName] nvarchar(20) null,
	[CountryCode] char(2) null
);

INSERT INTO Clients (
	[FirstName],
	[LastName],
	[CountryCode]
	)
VALUES
	(N'Adam', N'Smith', 'US'),
	(N'Belinda', N'Watson', 'CA'),
	(N'Charlotte', N'Duval', 'FR'),
	(N'Daniel', N'Ford', 'US');

IF TYPE_ID('CodeOfCountry') IS NOT NULL
	DROP TYPE CodeOfCountry;

CREATE TYPE CodeOfCountry AS TABLE
(
	Code char(2) not null
);
GO

-- EXEC ()
DECLARE @query_text nvarchar(max);

SET @query_text = N'
	SELECT *
	FROM Clients
	';

EXEC (@query_text);
GO

-- sp_execute_sql
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

-- simple dynamic sql query
SET @sql_text = N'
	SELECT *
	FROM Clients
	WHERE CountryCode = ''US''
	';

EXEC sp_executesql @sql_text;
GO

-- dynamic sql query with parameter
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

SET @sql_text = N'
	SELECT *
	FROM Clients
	WHERE CountryCode = @CountryCode
	';
SET @param_list = N'
	@CountryCode char(2)
	';

DECLARE @code char(2) = 'CA';

EXEC sp_executesql @sql_text, @param_list,
	@code;
GO

-- dynamic sql query with output parameter
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

SET @sql_text = N'
	SELECT @ClientCount = COUNT(*)
	FROM Clients
	WHERE CountryCode = @CountryCode
	';
SET @param_list = N'
	@CountryCode char(2),
	@ClientCount int OUTPUT
	';

DECLARE @TotalClients int;
DECLARE @code char(2) = 'FR';

EXEC sp_executesql @sql_text, @param_list,
	@code,
	@TotalClients OUTPUT;
SELECT @TotalClients AS [Total clients];
GO

-- dynamic sql query with table-valued parameter
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

SET @sql_text = N'
	SELECT *
	FROM Clients c
	JOIN @CountryList cl ON c.CountryCode = cl.Code
	';
SET @param_list = N'
	@CountryList CodeOfCountry READONLY
	';

DECLARE @CodeList CodeOfCountry;
INSERT INTO @CodeList (
	Code
	)
VALUES
	('CA'),
	('FR');

EXEC sp_executesql @sql_text, @param_list,
	@CodeList;
GO

-- dynamic sql query uses temp table
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

IF OBJECT_ID('tempdb..#codes') IS NOT NULL
	DROP TABLE #codes;

SELECT Code
INTO #codes
FROM (VALUES
	('CA'),
	('FR')
	) t(Code);	

SET @sql_text = N'
	SELECT *
	FROM Clients c
	JOIN #codes cl ON c.CountryCode = cl.Code
	';

EXEC sp_executesql @sql_text;
GO

-- dynamic sql query creates temp table
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

IF OBJECT_ID('tempdb..#codes') IS NOT NULL
	DROP TABLE #codes;

SET @sql_text = N'
	SELECT CountryCode
	INTO #codes
	FROM Clients
	';

EXEC sp_executesql @sql_text;
-- Oops! Invalid object name '#codes'.
SELECT * FROM #codes;
GO

-- pass too much parameters than used
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

SET @sql_text = N'
	SELECT *
	FROM Clients
	WHERE CountryCode = @CountryCode
	';
SET @param_list = N'
	@CountryCode char(2),
	@Name nvarchar(20)
	';

DECLARE @code char(2) = 'CA';

EXEC sp_executesql @sql_text, @param_list,
	@code,
	NULL;
GO

-- pass less parameters
DECLARE @sql_text nvarchar(max),
	@param_list nvarchar(max);

SET @sql_text = N'
	SELECT *
	FROM Clients
	WHERE CountryCode = @CountryCode
	';

-- Error! Must declare the scalar variable "@CountryCode".
EXEC sp_executesql @sql_text;
GO
